/*
 * Copyright (c) 1993 Christopher J. Kane.  All rights reserved.
 *
 * Permission to make and distribute copies of this software, and to use it
 * for any purpose, is hereby granted, provided this copyright and license
 * notice are preserved on all copies.  Modified versions of this software
 * must be plainly marked as such, and must not be misrepresented as being
 * the original software.  In no event may the original authorship of this
 * software be explicitly misrepresented.  Authors of software developed with
 * the aid of, or that incorporates, this software are under no obligation
 * to the copyright holder other than those obligations outlined in this
 * license; they need not credit the copyright holder nor the source of this
 * software.
 *
 * The copyright holder makes no express or implied warranty or representation
 * of any kind with respect to this software, including any warranty that this
 * software is error-free.  ANY IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS
 * FOR A PARTICULAR PURPOSE IS HEREBY DISCLAIMED.  IN NO EVENT WILL THE COPY-
 * RIGHT HOLDER BE LIABLE FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCI-
 * DENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE.
 * The copyright holder is under no obligation to provide any services, by way
 * of maintenance, update, or otherwise.
 *
 * Version: 1.0 (15 August 1993)
 */

#import "FindPanelClass.h"

@implementation Application (FindPanelClass)

static BOOL _FPloaded=NO;
static BOOL _fpLoaded=NO;

- (Class)findPanelClass
{
  static Class _findPanelClass=Nil;
  if (!_FPloaded)
    {
      char path[MAXPATHLEN+16];
      _FPloaded = YES;
      sprintf(path, "%s/FindPanel.bundle", [[NXBundle mainBundle] directory]);
      _findPanelClass = [[[NXBundle alloc] initForDirectory:path] classNamed:"FindPanel"];
      if ([delegate respondsTo:@selector(findPanelClassWasLoaded:)])
        _findPanelClass = [delegate findPanelClassWasLoaded:_findPanelClass];
    }
  return _findPanelClass;
}

- (BOOL)isFindPanelClassLoaded
{
  return _FPloaded;
}

- (FindPanel *)findPanel
{
  static FindPanel *_findPanel=nil;
  if (!_fpLoaded)
    {
      if ([delegate respondsTo:@selector(findPanelWillLoad:)])
        [delegate findPanelWillLoad:[self findPanelClass]];
      _findPanel = [[self findPanelClass] sharedInstance];
      if ([delegate respondsTo:@selector(findPanelDidLoad:)])
        _findPanel = [delegate findPanelDidLoad:_findPanel];
      _fpLoaded = YES;
    }
  return _findPanel;
}

- (BOOL)isFindPanelLoaded
{
  return _fpLoaded;
}

- enterSelection:sender
{
  return [[self findPanel] enterSelection:sender];
}

- findNext:sender
{
  return [[self findPanel] findForward:sender];
}

- findPrevious:sender
{
  return [[self findPanel] findBackward:sender];
}

- jumpToSelection:sender
{
  return [[self findPanel] jumpToSelection:sender];
}

- orderFrontFindPanel:sender
{
  return [[self findPanel] makeKeyAndOrderFront:sender];
}

@end
