// MODateTextCell.h
//
// by Mike Ferris
// Part of MOKit
// Copyright 1993, all rights reserved.


// ABOUT MOKit
// by Mike Ferris (mike@lorax.com)
//
// MOKit is a collection of useful and general objects.  Permission is 
// granted by the author to use MOKit in your own programs in any way 
// you see fit.  All other rights pertaining to the kit are reserved by the 
// author including the right to sell these objects as objects,  as part 
// of a LIBRARY, or as SOURCE CODE.  In plain English, I wish to retain 
// rights to these objects as objects, but allow the use of the objects 
// as pieces in a fully functional program.  Permission is also granted to 
// redistribute the source code of MOKit for FREE as long as this copyright 
// notice is left intact and unchanged.  NO WARRANTY is expressed or implied.  
// The author will under no circumstances be held responsible for ANY 
// consequences from the use of these objects.  Since you don't have to pay 
// for them, and full source is provided, I think this is perfectly fair.

// ABOUT MODateTextCell
//
// MODateTextCell is a subclass of MORegexTextCell that specifically checks
// dates.  This class was written because dates need just a little more 
// checking than a regular expression can supply.
//
// See the source for MORegexTextCell for more details about the 
// underlying mechanism.


#import "MOKit/MORegexTextCell.h"
#import "MOKit/MOString.h"

@interface MODateTextCell:MORegexTextCell
{
	BOOL		isEuropeanStyle;
	BOOL		doesFormat;
	MOString	*formatString;
}

+ initialize;

- initTextCell:(const char *)aString withRegex:(const char *)re;
- copyFromZone:(NXZone *)zone;
- free;

- (BOOL)isEuropeanStyle;
- setEuropeanStyle:(BOOL)flag;

- (BOOL)doesFormatDates;
- setFormatDates:(BOOL)flag;
- (const char *)formatString;
- setFormatString:(const char *)str;

- (BOOL)isEntryAcceptable:(const char *)aString;

- (int)day;
- (int)month;
- (int)year;

- endEditing:textObject;
- formatDate;

- setDoubleValue:(double)aDouble;
- setFloatValue:(float)aFloat;
- setIntValue:(int)anInt;
- setStringValue:(const char *)aString;
- setStringValueNoCopy:(const char *)aString;
- setStringValueNoCopy:(char *)aString shouldFree:(BOOL)flag;

- read:(NXTypedStream *)strm;
- write:(NXTypedStream *)strm;

- (const char *)getInspectorClassName;

@end
