# lib.make
# by Mike Ferris  (original by Art Isbell - art@cubisol.com)
# Part of MOKit
# Copyright 1993, all rights reserved

# ABOUT MOKit
# by Mike Ferris (mike@lorax.com)
#
# MOKit is a collection of useful and general objects.  Permission is 
# granted by the author to use MOKit in your own programs in any way 
# you see fit.  All other rights pertaining to the kit are reserved by the 
# author including the right to sell these objects as objects,  as part 
# of a LIBRARY, or as SOURCE CODE.  In plain English, I wish to retain 
# rights to these objects as objects, but allow the use of the objects 
# as pieces in a fully functional program.  Permission is also granted to 
# redistribute the source code of MOKit for FREE as long as this copyright 
# notice is left intact and unchanged.  NO WARRANTY is expressed or implied.  
# The author will under no circumstances be held responsible for ANY 
# consequences from the use of these objects.  Since you don't have to pay 
# for them, and full source is provided, I think this is perfectly fair.

# About lib.make
#
# This file defines several make targets that extend the Tool project 
# type in ProjectBuilder to support the building of libraries.  This 
# makefile leverages heavily off the Next makefile architecture.
#
# This file is my modifications to the original file by Art Isbell.

LIBTOOL = /bin/libtool

lib::
	@($(MKDIRS) $(SYMROOT) ; \
 	$(process_target_archs)_obj ; \
	if [ "$(ENGLISH)" = "YES" -o "$(JAPANESE)" != "YES" ] ; then \
		language_cflags="" ; \
	else \
		language_cflags="-DKANJI" ; \
	fi ; \
	$(MAKE) libproject \
		"PRODUCT_ROOT = $(SYMROOT)" \
		"LIBPRODUCT = $(LOCALLIBDIR)/lib$(NAME).a" \
		"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
		"PROJ_CFLAGS = $(NORMAL_CFLAGS) $$language_cflags" \
		"APP_MAKEFILE_DIR = $(APP_MAKEFILE_DIR)" \
		"RC_CFLAGS = $$arch_cflags" \
		"RC_ARCHS = $$archs" \
		"SRCROOT = $(SRCROOT)" \
		"OBJROOT = $(OBJROOT)" \
		"SYMROOT = $(SYMROOT)/sym" \
		"SYM_DIR = $(SYMROOT)/sym" )

debuglib::
	@($(MKDIRS) $(SYMROOT) ; \
 	$(process_target_archs)_debug_obj ; \
	if [ "$(ENGLISH)" = "YES" -o "$(JAPANESE)" != "YES" ] ; then \
		language_cflags="" ; \
	else \
		language_cflags="-DKANJI" ; \
	fi ; \
	$(MAKE) libproject \
		"PRODUCT_ROOT = $(SYMROOT)" \
		"LIBPRODUCT = $(LOCALLIBDIR)/lib$(NAME)_g.a" \
		"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
		"PROJ_CFLAGS = $(DEBUG_CFLAGS) $$language_cflags" \
		"APP_MAKEFILE_DIR = $(APP_MAKEFILE_DIR)" \
		"RC_CFLAGS = $$arch_cflags" \
		"RC_ARCHS = $$archs" \
		"SRCROOT = $(SRCROOT)" \
		"OBJROOT = $(OBJROOT)" \
		"SYMROOT = $(SYMROOT)/sym" \
		"SYM_DIR = $(SYMROOT)/sym" )

profilelib::
	@($(MKDIRS) $(SYMROOT) ; \
 	$(process_target_archs)_profile_obj ; \
	if [ "$(ENGLISH)" = "YES" -o "$(JAPANESE)" != "YES" ] ; then \
		language_cflags="" ; \
	else \
		language_cflags="-DKANJI" ; \
	fi ; \
	$(MAKE) libproject \
		"PRODUCT_ROOT = $(SYMROOT)" \
		"LIBPRODUCT = $(LOCALLIBDIR)/lib$(NAME)_p.a" \
		"OFILE_DIR = $(OBJROOT)/$$obj_dir" \
		"PROJ_CFLAGS = $(PROFILE_CFLAGS) $$language_cflags" \
		"APP_MAKEFILE_DIR = $(APP_MAKEFILE_DIR)" \
		"RC_CFLAGS = $$arch_cflags" \
		"RC_ARCHS = $$archs" \
		"SRCROOT = $(SRCROOT)" \
		"OBJROOT = $(OBJROOT)" \
		"SYMROOT = $(SYMROOT)/sym" \
		"SYM_DIR = $(SYMROOT)/sym" )

libproject:: $(INITIAL_TARGETS) toplevel_subcomponents resources $(LIBPRODUCT)

$(LOCALLIBDIR)/lib$(NAME).a: $(PRODUCT_DEPENDS)
	@(if [ "`$(ECHO) $(OFILES) $(OTHER_OFILES) | wc -w`" != "       0" ] ; \
	then \
		$(STRIP) -x $(OFILES) $(OTHER_OFILES); \
		cmd="$(LIBTOOL) -o $(LIBPRODUCT) $(OFILES) $(OTHER_OFILES)"; \
		echo $$cmd; \
		$$cmd; \
	fi	)

$(LOCALLIBDIR)/lib$(NAME)_g.a : $(PRODUCT_DEPENDS)
	@(if [ "`$(ECHO) $(OFILES) $(OTHER_OFILES) | wc -w`" != "       0" ] ; \
	then \
		cmd="$(LIBTOOL) -o $(LIBPRODUCT) $(OFILES) $(OTHER_OFILES)"; \
		echo $$cmd; \
		$$cmd; \
	fi	)

$(LOCALLIBDIR)/lib$(NAME)_p.a: $(PRODUCT_DEPENDS)
	@(if [ "`$(ECHO) $(OFILES) $(OTHER_OFILES) | wc -w`" != "       0" ] ; \
	then \
		cmd="$(LIBTOOL) -o $(LIBPRODUCT) $(OFILES) $(OTHER_OFILES)"; \
		echo $$cmd; \
		$$cmd; \
	fi	)
